<?php

namespace Spec\Infomaker\Everyware\Twig;

use Infomaker\Everyware\Twig\ViewSetup;
use PhpSpec\ObjectBehavior;
use Twig\Environment;
use Twig\Extension\ExtensionInterface;
use Twig\Loader\FilesystemLoader;
use Twig\NodeVisitor\NodeVisitorInterface;
use Twig\RuntimeLoader\RuntimeLoaderInterface;
use Twig\TokenParser\TokenParserInterface;
use Twig\TwigFilter;
use Twig\TwigFunction;
use Twig\TwigTest;

/**
 * @method registerTwigFolder(string $namespace, string $path)
 * @method render(string $template)
 * @method view(string $template)
 * @method addFilter(string $name, string $callable, array $options)
 * @method addExtension(ExtensionInterface $extension)
 * @method addFunction(string $name, string $callable, array $options)
 * @method addGlobal(string $key, string $value)
 * @method addNodeVisitor(NodeVisitorInterface $visitor)
 * @method addRuntimeLoader(RuntimeLoaderInterface $loader)
 * @method addTokenParser(TokenParserInterface $parser)
 * @method addTest(string $name, string $callable, array $options)
 * @method addWpFunction(string $callable,string $name = null)
 */
class ViewSetupSpec extends ObjectBehavior
{
    /**
     * @var Environment
     */
    private $environment;

    function let(Environment $environment)
    {
        $this->environment = $environment;
        $this->beConstructedWith($environment);
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(ViewSetup::class);
    }

    function it_can_register_template_directories_on_to_the_loader(FilesystemLoader $loader)
    {
        $path = __DIR__;
        $namespace = 'namespace';

        $this->environment->getLoader()
            ->willReturn($loader);

        $loader->addPath($path, $namespace)->shouldBeCalled();
        $this->environment->setLoader($loader)->shouldBeCalled($loader);

        $this->registerTwigFolder($namespace, $path);
    }

    function it_will_not_register_directories_twice(FilesystemLoader $loader)
    {
        $path = __DIR__;
        $namespace = 'namespace';

        $this->environment->getLoader()
            ->shouldBeCalledTimes(1)
            ->willReturn($loader);

        $loader->addPath($path, $namespace)->shouldBeCalledTimes(1);
        $this->environment->setLoader($loader)->shouldBeCalledTimes(1);

        $this->registerTwigFolder($namespace, $path);
        $this->registerTwigFolder($namespace, $path);
    }

    function it_will_not_register_directoriesthat_cant_be_found(FilesystemLoader $loader)
    {
        $path = __DIR__ . '/missing_dir';
        $namespace = 'namespace';

        $this->environment->getLoader()->shouldNotBeCalled();
        $loader->addPath($path, $namespace)->shouldNotBeCalled();
        $this->environment->setLoader($loader)->shouldNotBeCalled();

        $this->registerTwigFolder($namespace, $path);
    }

    public function it_should_render_template()
    {
        $template = '@template/name';
        $content = [
            'content' => 'Test data'
        ];

        $this->environment->display($template, $content)->shouldBeCalled();

        $this->render($template, $content);
    }

    public function it_should_be_able_to_render_template_without_content()
    {
        $template = '@template/name';

        $this->environment->display($template, [])->shouldBeCalled();

        $this->render($template);
    }

    public function it_should_view_template()
    {
        $template = '@template/name';
        $content = [
            'content' => 'Test data'
        ];

        $view = 'Generated Template';

        $this->environment->render($template, $content)->willReturn($view);

        $this->view($template, $content)->shouldReturn($view);
    }

    public function it_should_be_able_to_view_template_without_content()
    {
        $template = '@template/name';
        $view = 'Generated Template';

        $this->environment->render($template, [])->willReturn($view);

        $this->view($template)->shouldReturn($view);
    }

    public function it_adds_Extensions_to_the_environment(ExtensionInterface $extension)
    {
        $this->environment->addExtension($extension)->shouldBeCalled();

        $this->addExtension($extension);
    }

    public function it_adds_Filters_to_the_environment()
    {
        $name = 'name';
        $callable = 'callable';
        $options = [];
        $this->environment->addFilter(new TwigFilter($name, $callable, $options))->shouldBeCalled();

        $this->addFilter($name, $callable, $options);
    }

    public function it_adds_Functions_to_the_environment()
    {
        $name = 'name';
        $callable = 'callable';
        $options = [];
        $this->environment->addFunction(new TwigFunction($name, $callable, $options))->shouldBeCalled();

        $this->addFunction($name, $callable, $options);
    }

    public function it_adds_Global_variables_to_the_environment()
    {
        $key = 'name';
        $value = 'test value';

        $this->environment->addGlobal($key, $value)->shouldBeCalled();

        $this->addGlobal($key, $value);
    }

    public function it_adds_NodeVisitors_to_the_environment(NodeVisitorInterface $visitor)
    {
        $this->environment->addNodeVisitor($visitor)->shouldBeCalled();

        $this->addNodeVisitor($visitor);
    }

    public function it_adds_RuntimeLoaders_to_the_environment(RuntimeLoaderInterface $loader)
    {
        $this->environment->addRuntimeLoader($loader)->shouldBeCalled();

        $this->addRuntimeLoader($loader);
    }

    public function it_adds_Tests_to_the_environment()
    {
        $name = 'name';
        $callable = 'callable';
        $options = [];

        $this->environment->addTest(new TwigTest($name, $callable, $options))->shouldBeCalled();

        $this->addTest($name, $callable, $options);
    }

    public function it_adds_TokenParsers_to_the_environment(TokenParserInterface $parser)
    {
        $this->environment->addTokenParser($parser)->shouldBeCalled();

        $this->addTokenParser($parser);
    }

    public function it_adds_WpFunctions_to_the_environment()
    {
        $callable = 'wp_function';
        $this->environment->addFunction(new TwigFunction($callable, $callable, []))->shouldBeCalled();

        $this->addWpFunction($callable);
    }

    public function it_adds_WpFunctions_to_the_environment_under_different_name()
    {
        $name = 'name';
        $callable = 'wp_function';
        $this->environment->addFunction(new TwigFunction($name, $callable, []))->shouldBeCalled();

        $this->addWpFunction($callable, $name);
    }

    public function it_can_determine_if_an_extension_has_been_added()
    {
        $extensionClass = 'Extension\Name';
        $this->environment->hasExtension($extensionClass)->willReturn(true);

        $this->hasExtension($extensionClass)->shouldReturn(true);
    }
}
